package nz.ac.canterbury.debugassist.model;
import java.util.*;
/**
 * @author  James
 */
public class Variable implements Comparable<Variable> {
	private HashMap<String, Variable> refs;
	
	private String type;

	private String name;

	private String data;
	
	private int updateCount;
	private boolean isRoot;

	private boolean updated;
	public Variable(String type, String name, String data, boolean isRoot, boolean updated) {
		this.type = type;
		this.name = name;
		this.data = data;
		this.isRoot = isRoot;
		updateCount = 0;
		this.updated = updated;
		refs = new LinkedHashMap<String, Variable>();
	}

	public boolean isUpdated() { return updated;}
	
	public void addVariable(Variable v) {
		refs.put(v.getName(), v);
	}
	public void incrementUpdateByOne() {
		updateCount++;
	}
	public boolean displayable() {
		return !(type.equals("Root") && name.equals("Root"));   
	}
	public void deleteVariable(String name) {
		refs.remove(name);
	}
	public void updateData(String data) {
		this.data = data;
		updateCount++;
	}
	public Variable[] getAllChildren() {

		return (Variable[]) refs.values().toArray(new Variable[0]);
	}
	
	public HashMap<String, Variable> getHashMapChildren() {

		return refs;
	
	}
	public boolean isLeaf() {
		return refs.size() == 0;
	}
	public void updateVariableCount(int num) {
		updateCount = num;
	}

	public String getData() { return data;}
	
	public String getType() { return type;}

	public String getName() { return name;}

	public int getUpdateCount() { return updateCount;}
	public boolean hasChild(String name) { return refs.containsKey(name);}
	public Variable getChild(String name) { return refs.get(name);}
	public String toString() {
		return "Variable (" + type +"): " + name + ":" + data + " ["+ updateCount +"]";
	}
	public HashMap<String, Variable> getChildren() {

		return refs;
	}
	
	public static HashMap<String, Variable> arrayToMap(Variable[] array) {
		HashMap<String, Variable> results = new HashMap<String, Variable>();
		for(Variable v : array) {
			results.put(v.getName(), v);
		}
	
		return results;
	}
	public String getShortName() {
		return getName() + " [" + getType() + "] " + getData();
	}
	
	public int getObjectID() {
		int dataSubStringPoint = data.indexOf("(id=");
		if(dataSubStringPoint== -1) {
			return -1;
		}
		String subEndData = data.substring(dataSubStringPoint+4);
		String numberData = subEndData.substring(0,subEndData.indexOf(')'));
		try { 
			return Integer.parseInt(numberData); 
		} catch(Exception ex) { 
			return -1;
		}
		
	}
	
	@Override
	public int compareTo(Variable o) {
		
		return this.name.compareTo(o.name);
	}
	
	public void removePrimitives() {
		for(String ref : ((HashMap<String, Variable>)refs.clone()).keySet()) {
			Variable v = refs.get(ref);
			if(v.isLeaf()) {
				refs.remove(ref);
			}
		}
	}
	
	public void removeAllChildren() {
		refs.clear();
	}
	
}
